#!/bin/bash
#

# this script builds both ifdhc and ifdhc_config
# need an in source build

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <<e15:p2714b>|<e17:<p2714b|p2715a|p372>>>|<e19:<p2715a|p372>>|<c[27]:<p2714b|p2715a|p372>>> <debug|prof> [tar]"
   echo "        p372    -> python v3_7_2"
   echo "        p2715a  -> python v2_7_15a"
   echo "        p2714b  -> python v2_7_14b"
   echo "        c7  -> Clang v7_0_0 / C++17 / libc++"
   echo "        c2  -> Clang v5_0_1 / C++17 / libc++"
   echo "        e19 -> gcc v8_2_0 / C++17"
   echo "        e17 -> gcc v7_3_0 / C++17"
   echo "        e15 -> gcc v6_4_0 / C++14"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}

    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims"
       exit 1
    fi

}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=ifdhc
origpkgver=v2_5_4
pkgver=${origpkgver}
ssibuildshims_version=v1_04_14
pkgdotver=`echo ${pkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarver=`echo ${origpkgver} | sed -e 's/_/./g'`
pkgtarfile=${package}-${origpkgver}.tar 

get_this_dir

get_ssibuildshims

source define_basics --

pkgups=${product_dir}/${package}/${pkgver}/ups

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   cd ${product_dir}
   tar cjf ifdhc_config-${pkgdotver}-noarch.tar.bz2 ifdhc_config/${pkgver} ifdhc_config/${pkgver}.version ifdhc_config/current.chain
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
mkdir -p ${pkgups}
if [ ! -d ${pkgups} ]
then
   echo "ERROR: failed to create ${pkgups}"
   exit 1
fi

cd ${pkgdir} 
tar xf ${tardir}/${pkgtarfile} || ssi_die "ERROR: failed to unwind ${pkgtarfile}"
patch -b -p0 < "${patchdir}/ifdhc.patch" || ssi_die "ERROR: application of patch ${patchdir}/ifdhc.patch failed"
cp -p ${pkgdir}/ups/ifdhc.table ${pkgups}/ || ssi_die "ERROR: copy of ups table file failed"

echo
if  ups exist ifdhc_config ${pkgver} -z ${product_dir} >/dev/null 2>&1; then
   echo "ifdhc_config ${pkgver} is already installed"
else
   echo "install ifdhc_config ${pkgver}"
   ${pkgdir}/ups/install_ifdhc_config.sh ${product_dir} ${pkgver} || ssi_die "ERROR: install of ifdhc_config failed"
fi
#ups list -aK+ ifdhc_config ${pkgver} -z ${product_dir}

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "ERROR: fake setup failed"


if [ -z ${PYTHONHOME} ]
then
  echo "ERROR: failed to setup python"
  exit 1
else
  echo
  echo "PYTHONHOME is ${PYTHONHOME}"
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

qlist=(`echo ${basequal} | sed -e 's/:/ /g' | tr " " "\n"| sort |tr "\n" " "`)
for qual in "${qlist[@]}"; do
  case $qual in
    e1[0145])
      cqual=$qual
      cc=gcc
      cxx=g++
      stdcxx="-std=c++14"
      ;;
    e1[79])
      cqual=$qual
      cc=gcc
      cxx=g++
      stdcxx="-std=c++17"
      ;;
    c[27])
      cqual=$qual
      cc=clang
      cxx=clang++
      stdcxx="-std=c++17"
      ;;
    p2714b|p2715a|p372)
      pqual=$qual
      ;;
    *)
      ssi_die "Unrecognized qualifier component $qual"
  esac
done
if [[ -z "$cqual" ]]; then
  ssi_die "Base qualifier is mandatory"
fi
if [[ -z "$pqual" ]]; then
  ssi_die "A python qualifier is mandatory."
fi

case ${cqual}:${pqual} in
e1[57]:p2714b|e1[79]:p2715a)
 ;;
c2:p2714b|c[27]:p2715a)
 ;;
e1[79]:p372|c[27]:p372)
 ;;
*) 
   echo "ERROR: unrecognized qualifier combination ${cqual}:${pqual}"
   usage
   exit 1
esac

case ${extraqual} in
debug) ARCH="${stdcxx} -O0 -g";;
prof)  ARCH="${stdcxx} -O3 -g -DNDEBUG ";;
esac
export ARCH

export PYTHON_CONF=`echo $PYTHON_LIB/python*/config`

echo

set -x

cd ${pkgdir} 

env CC=${cc} CXX=${cxx} make IFDHC_VERSION=${origpkgver} all || ssi_die "ERROR: make all failed"
env CC=${cc} CXX=${cxx} make install || ssi_die "ERROR: make install failed"
#env CC=${cc} CXX=${cxx} make clean || ssi_die "ERROR: make clean failed"

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi


# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
echo "unsetup"
unsetup ${package}
echo "setup"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS} || ssi_die "ERROR: setup failed"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   cd ${product_dir}
   tar cjf ifdhc_config-${pkgdotver}-noarch.tar.bz2 ifdhc_config/${pkgver} ifdhc_config/${pkgver}.version  ifdhc_config/current.chain

fi

exit 0
